-- ----------------------------------
-- (c) WolfVision Innovation GmbH
-- VHDL Komponente fr Altera Avalon 
-- Memory Mapped Slave
--
-- ISP1761 Interface fr SOPC
--
-- 19.03.2008, GLo: Erste Version in VHDL
-- ----------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.all;

entity isp1761_if IS
	PORT (
		-- Global Avalon interface signals   
		csi_clockreset_clk						: in std_logic;
		csi_clockreset_reset_n					: in std_logic;
		-- Signals for Avalon-MM slave port
		avs_s1_write_n			: in std_logic;
		avs_s1_writedata		: in std_logic_vector(31 downto 0);
		avs_s1_chipselect_n		: in std_logic;
		avs_s1_address			: in std_logic_vector(15 downto 0);
		avs_s1_read_n			: in std_logic;
		avs_s1_readdata			: out std_logic_vector(31 downto 0);
		--avs_s1_waitrequest		: out std_logic;
--		avs_s1_irq_n			: out std_logic;
--		avs_s2_irq_n			: out std_logic;
		avs_s1_irq_n			: out std_logic;
        -- ISP1761 signals, export
		iHC_DACK		: in std_logic;
		iDC_DACK		: in std_logic;
		iHC_SUSPEND		: in std_logic;
		iDC_SUSPEND		: in std_logic;
		ioISP_DATA		: inout std_logic_vector(31 downto 0);
		oDC_DREQ		: out std_logic;
		oHC_DREQ		: out std_logic;
		oHC_SUSPEND		: out std_logic;
		oDC_SUSPEND		: out std_logic;
		oISP_ADDR		: out std_logic_vector(15 downto 0);
		oISP_CS_N		: out std_logic;
		oISP_RD_N		: out std_logic;
		oISP_WR_N		: out std_logic;
		iISP_DC_IRQ		: in std_logic;
		iISP_HC_IRQ		: in std_logic;
		iISP_DC_DREQ	: in std_logic;
		iISP_HC_DREQ	: in std_logic;
		oISP_HC_DACK	: out std_logic;
		oISP_DC_DACK	: out std_logic;
		ioISP_HC_SUSPEND: inout std_logic;
		ioISP_DC_SUSPEND: inout std_logic
	);
end isp1761_if;

architecture rtl OF isp1761_if IS

--signal write_cycle		: boolean;
--signal write_cycle_old	: boolean;
--signal read_cycle		: boolean;
--signal read_cycle_old	: boolean;
--signal writedata		: std_logic_vector(out_width-1 downto 0);
--signal readdata			: std_logic_vector(out_width-1 downto 0);

begin

--write_cycle <= avs_s1_chipselect='1' and avs_s1_write_n='0' and (avs_s1_address="00");		-- write_cycle wird '1' bei schreibzugriff auf pipe_if
--read_cycle <= avs_s1_chipselect='1' and avs_s1_read_n='0' and (avs_s1_address="00");		-- read_cycle wird '1' bei schreibzugriff auf pipe_if

--pipe_if_proc:	process (csi_clockreset_clk, csi_clockreset_reset_n)
--begin
--	if csi_clockreset_reset_n = '0' then
--		oCS <= '0';
--		oWRITE_N <= '1';
--		oREAD_N <= '1';
--		read_cycle_old <= false;
--		write_cycle_old <= false;
--		writedata(out_width-1 downto 0) <= (others => '0');
--		readdata(out_width-1 downto 0) <= (others => '0');
--	elsif rising_edge(csi_clockreset_clk) then 	
--		if write_cycle then
--		end if;
--	end if;
--end if;

ioISP_DATA		<= (others=>'Z') when avs_s1_write_n='1' else avs_s1_writedata;
avs_s1_readdata <= ioISP_DATA;
--avs_s1_irq_n	<= iISP_HC_IRQ;
--avs_s2_irq_n	<= iISP_DC_IRQ;
avs_s1_irq_n	<= iISP_HC_IRQ and iISP_DC_IRQ;
oDC_DREQ		<= iISP_DC_DREQ;
oHC_DREQ		<= iISP_HC_DREQ;
oISP_ADDR 		<= avs_s1_address;
oISP_CS_N		<= avs_s1_chipselect_n;
oISP_RD_N 		<= avs_s1_read_n;
oISP_WR_N		<= avs_s1_write_n;
oISP_HC_DACK	<= iHC_DACK;
oISP_DC_DACK	<= iDC_DACK;
ioISP_HC_SUSPEND<= 'Z'; -- when avs_s1_write_n='1' else iHC_SUSPEND;
oHC_SUSPEND		<= ioISP_HC_SUSPEND;
ioISP_DC_SUSPEND<= 'Z'; -- when avs_s1_write_n='1' else iDC_SUSPEND;
oDC_SUSPEND		<= ioISP_DC_SUSPEND;

end rtl;
