/**********************************************************************
 * Philips Mass Storage disk emulation
 *
 * (c) 2002 Koninklijke Philips Electronics N.V., All rights reserved
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * File Name:   disk_emu.h
 *
 * Refering linux kernel version 2.6.9
 *
 * History:     
 *
 * Date                Author                  Comments
 * ---------------------------------------------------------------------
 * Nov 29 2005        Prabhakar Kalasani      Initial Creation
 *
 ************************************************************************/
#ifndef __DISK_EMU_H__
#define __DISK_EMU_H__

/* Disk Emulation file */
#define         MSDISK_FILE                                     "/mnt/nandflash_img/usbdisk"
#define         _8MB_SIZE_

#ifdef          _8MB_SIZE_
#define         LOGICAL_BLOCKS                                  0x00004000	   //!!! 0x00004000 jetzt nur noch 2 MB fr Tests
#define         LINUX_UNIT_SIZE                                 16
#else
#define         LOGICAL_BLOCKS                                  0x00008000
#define         LINUX_UNIT_SIZE                                 32
#endif /* _8MB_SIZE_ */

#define         BLOCK_SIZE                                      0x00000200
#define         LAST_LBA                                        (LOGICAL_BLOCKS-1)

#define         INQUIRY_DATA_SIZE                               0x24
#define         READ_CAPACITY_DATA_SIZE                         0x08
#define         MODE_SENSE_DATA_SIZE                            0x0C
#define         REQUEST_SENSE_DATA_SIZE                         0x12

#define         CMD_TEST_UNIT_READY                             0x00
#define         CMD_REQUEST_SENSE                               0x03
#define         CMD_FORMAT_UNIT                                 0x04
#define         CMD_INQUIRY                                     0x12
#define         CMD_RESERVE                                     0x16
#define         CMD_RELEASE                                     0x17
#define         CMD_MODE_SENSE                                  0x1A
#define         CMD_START_STOP_UNIT                             0x1B
#define         CMD_SEND_DIAGNOSTIC                             0x1D
#define         CMD_MEDIUM_REMOVAL                              0x1E
#define         CMD_READ_CAPACITY                               0x25
#define         CMD_READ10                                      0x28
#define         CMD_WRITE10                                     0x2A
#define         CMD_VERIFY10                                    0x2F

/* 
 * Definitions for INQUIRY CMD fields 
 */
#define         INQUIRY_EVPD                                    0x01
/* 
 * Definitions of READ_CAPACITY fields 
 */
#define         READ_CAPACITY_RMI                               0x01
/* 
 * Definition of RESERVE RELEASE fields 
 */
#define         RESERVE_EXTENT                                  0x01


/* REQUEST_SENSE */
/* 
 * Sense Key definitions 
 */
#define         SENSE_KEY_NO_SENSE                              0x00
#define         SENSE_KEY_RECOVERED_ERROR                       0x01
#define         SENSE_KEY_NOT_READY                             0x02
#define         SENSE_KEY_MEDIUM_ERROR                          0x03
#define         SENSE_KEY_HARDWARE_ERROR                        0x04
#define         SENSE_KEY_ILLEGAL_REQUEST                       0x05
#define         SENSE_KEY_UNIT_ATTENTION                        0x06
#define         SENSE_KEY_DATA_PROTECT                          0x07

/* 
 * Additional Sense Codes (ASC) 
 */
#define         ASC_ADDR_NOT_FOUND_4_ID_FLD                     0x12
#define         ASC_ADDR_NOT_FOUND_4_DATA_FLD                   0x13
#define         ASC_INV_FIELD_IN_CDB                            0x24
#define         ASC_MEDIUM_CHANGE                               0x28

/* 
 * Additional Sense Code Qualifiers (ASCQ) 
 */
#define         ASCQ_INV_FIELD_IN_CDB                           0x00
#define         MS_DEV_FILE_NAME                                "/dev/usbdev"

/* OTG ms disk status */
typedef struct disk_emu_info {
    unsigned char       prevent;                                /* Allow/prevent medium removal */
    unsigned char       sense_key;                              /* Sense key */
    unsigned char       asc;                                    /* Additional sense code */
    unsigned char       ascq;                                   /* Additional sense code qualifier */
} disk_emu_info_t;

/*
 * Disk emulation states
 */
#define         DISK_EMU_IDLE                                   0x00
#define         DISK_EMU_CMD                                    0x01
#define         DISK_EMU_STATUS                                 0x02
#define         DISK_EMU_DATA_IN                                0x03
#define         DISK_EMU_DATA_OUT                               0x04

#endif /* __DISK_EMU_H__ */
