/*******************************************************************
 * Philips USB Mass Storage Class driver (Device side)
 *
 * (c) 2002 Koninklijke Philips Electronics N.V., All rights reserved
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * File Name:   devmscd.h
 *
 * Refering linux kernel version 2.6.9
 *
 * History:
 *
 * Date            Author                 Comments
 * -----------------------------------------------------------------
 * Nov 29 2005    Prabhakar Kalasani      Initial Creation
 *
 *******************************************************************/

#ifndef __DEVMSCD_H__
#define __DEVMSCD_H__

/* 
 * Mass Storage (BOT) class commands
 */
#define         DEVMSCD_GET_MAX_LUN                     0xFE
#define         DEVMSCD_MS_RESET                        0xFF

/*
 * Mass storage class sates
 * State Transistion
 *                           | --------------------
 *                          IDLE                  |
 *                           |                    |
 *                          CBW                  |
 *                           |                    |
 *                     ----------------           |
 *                     |              |           |
 *                  DATA_IN        DATA_OUT      |
 *                     |              |           |
 *                     ----------------           |
 *                            |                   |
 *                           CSW                 |
 *                            |--------------------
 */
#define         DEVMSCD_IDLE                            0x00            /* Idle, no command */
#define         DEVMSCD_CBW                             0x01            /* Received command */
#define         DEVMSCD_CSW                             0x02            /* Status Stage */
#define         DEVMSCD_DATA_IN                         0x03            /* DATA IN stage */
#define         DEVMSCD_DATA_OUT                        0x04            /* DATA OUT State */

/*
 * message lengths
 */
#define         DEVMSCD_CBW_LENGTH                      0x1F            /* CBW length */
#define         DEVMSCD_CSW_LENGTH                      0x0D            /* CSW length */
#define         DEVMSCD_CBW_WRAPPER                     0x0F            /* CBWCB start point */

/*
 * Signatures (little Indian)
 */
#define         DEVMSCD_CBWSIGNATURE                    0x43425355              /* CBW Signature */
#define         DEVMSCD_CSWSIGNATURE                    0x53425355              /* CSW Signature */

/*
 * CBW Flag vlaues
 */
#define         DEVMSCD_CBWFLAG_OUT                     0x00            /* Data out from Host */
#define         DEVMSCD_CBWFLAG_IN                      0x80            /* Data In to Host */

typedef struct devmscd_device {
    __u8                reserved1;                          
    __u8                state;                          /* Mass storage BOT state */
    __u8                data_in_pipe;                   /* Bulk In pipe (DATA IN) */
    __u8                data_out_pipe;                  /* Bulk Out Pipe (DATA OUT) */
    __u32               tx_residue;                             /* Transfer residue */
    __u8                status_queue;                   /* command status queue flag */
    __u8                config;                                 /* USB Configuration # */
    __u8                reserved2;                          
    __u8                reserved3;                          

    __u8                cbw[DEVMSCD_CBW_LENGTH];        /* Command Block Word */
    __u8                reserved4;                          
    __u8                csw[DEVMSCD_CSW_LENGTH];        /* Command Status Word */
    __u8                reserved5;                          
} __attribute__ ((aligned(4))) devmscd_dev_t;


/* 
 * Driver definitions
 */
#define DRIVER_AUTHOR           "Philips Semiconductors"
#define DRIVER_DESC             "USB Mass storage Class Driver (Device)"
#define DEVMSCD_VERSION         "1.0"
#define DRIVER_NAME             "devmscd"

#include "../../hal/hal_intf.h"

#endif /* __DEVMSCD_H__ */
