/*************************************************************
 * Philips USB device protocol and Configuration driver
 *
 * (c) 2002 Koninklijke Philips Electronics N.V., All rights reserved
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * File Name:   pdc_bus.c
 *
 * Refering linux kernel version 2.6.9
 *
 * History:     
 *
 * Date              Author                   Comments
 * --------------------------------------------------------------
 * Oct 4, 2004        SRIRAM PAULRAJ          Initial Creation
 * Oct 10, 2004       SRIRAM PAULRAJ          Clear feature on 
 *                                            unknown wIndex is stalled 
 * Nov 3, 2004        SRIRAM PAULRAJ          Support USB 2.0
 * May 5, 2006        Grace Baldonasa          Added MTP support
 *
 ****************************************************************/
#ifdef LINUX_269
#include <linux/config.h>
#endif
#include <linux/types.h>
#include <linux/string.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/timer.h>
#include <linux/mm.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>
#include <linux/major.h>
#include <linux/errno.h>
#include <linux/slab.h>
#include <linux/module.h>
#include <linux/vmalloc.h>
#include <linux/init.h>
#include <linux/usb.h> 

#include <asm/byteorder.h>
#include <asm/irq.h>
#include <asm/segment.h>
#include <asm/io.h>
#include <linux/proc_fs.h>
#include <linux/poll.h>

#include "pdc_intf.h"
#include "usb_pdc.h"
#include "pdc_protocol.h"
#include "pdc_bus.h"
#include "../hal/hal_intf.h"
#include "../hal/isp1761.h"

#ifdef CONFIG_USB_ISP1761_MSCD
#include "usb_desc_mscd.c"
#endif
#if defined(CONFIG_USB_ISP1761_WOLFDEV) || defined(CONFIG_USB_ISP1761_WOLFDEV_MODULE)
#include "usb_desc_wolfdev.c"
#endif

//extern __u32 pdc_read32(__u16 reg);
//extern void pdc_write32(__u16 reg, __u32 data);

/*------------------------------------------------------------------*
 *                   Local variables                                *
 *------------------------------------------------------------------*/
struct pdc_bus                  pdc_bus_device;
struct pdc_bus                  *bus_dev = &pdc_bus_device;
struct pdc_pipe_opr             ctrl_pipe_opr;
LIST_HEAD(pdc_class_drv_list);

extern u8       HS_Dev_flag;    


/*-------------------------------------------------------------------*
 *                     Local Function Declerations                   *
 *-------------------------------------------------------------------*/
static  void pdc_bus_stall_control_pipe(void);
static  void pdc_bus_dev_notif(unsigned long    context, unsigned long notif);
static  int pdc_bus_ctrl_pipe_notificaion(unsigned long notif_type, void *priv,unsigned char    *cmd);
static  void pdc_bus_get_descriptor(__u8 *command);
static  int     pdc_notif_write_control_pipe(__u8       *buff, __u16    len);
EXPORT_SYMBOL(pdc_notif_write_control_pipe); 
static  int     pdc_notif_read_control_pipe(__u8        *buff, __u16    len, void *callback);
EXPORT_SYMBOL(pdc_notif_read_control_pipe);
static  void pdc_bus_usb_feature(__u8   *command);
static  void pdc_bus_get_status(__u8    *command);
static  void pdc_bus_set_configuration(unsigned char *command);
static  void pdc_bus_set_interface(unsigned char *command);





/*----------------------------------------------------------------*
 *                   External Interface Functions                 *
 *----------------------------------------------------------------*/

/* 
 * Class Driver registration/Deregistration functions 
 */
int pdc_register_class_drv( struct pdc_class_drv *drv)
{
    dev_print("++ pdc_register_class_drv(%p)\n",drv);

    /* 
     * Put it in the class drivers list
     */
    list_add(&drv->driver_list, &pdc_class_drv_list);

    dev_print(KERN_INFO "ISP1761 Registered Device Class Driver \n",drv->name);

    /* TODO if the device is active, call the probe function */


    return 0;   /* Success */
} /* End of pdc_register_class_drv() */

EXPORT_SYMBOL(pdc_register_class_drv);

void pdc_deregister_class_drv( struct pdc_class_drv *drv)
{
    dev_print("++ pdc_deregister_class_drv(%p)\n",drv);

    list_del(&drv->driver_list);

    if(drv && drv->set_config) {
        drv->set_config(drv->priv_data,0);
    }

    printk(KERN_INFO __FILE__ ": De-registered Driver %s\n",drv->name);


    return;
} /* End of pdc_deregister_class_drv() */

EXPORT_SYMBOL(pdc_deregister_class_drv);

/* Initialize all the end point data structures */
int     pdc_bus_init(struct pdc_dev     *pdc) 
{
    /* Enable only control end points */
    __u16       size;
    struct pdc_ep_desc  *ep_desc;
    int         result;

    dev_print("++ pdc_bus_init(void)\n");

    bus_dev->ctrl_pipe = PDC_INV_PIPE_HANDLE;
    INIT_LIST_HEAD(&pdc_class_drv_list);

    bus_dev->configuration = 0;
    bus_dev->interface = 0;

    bus_dev->state = PDC_BUS_UNKNOWN;

    size = sizeof(struct pdc_config_dev) + (sizeof(struct pdc_ep_desc) * PDC_MAX_PIPES);
    bus_dev->pdc_config = (struct pdc_config_dev*) kmalloc(size, GFP_KERNEL);

    if(!(bus_dev->pdc_config)) {
        return -1;
    }

    bus_dev->pdc_config->num_eps = 3;   /* Control+Bulk In + Bulk OUT */

    ep_desc = bus_dev->pdc_config->ep_desc;

    ep_desc->ep_num = 0;                                                /* Control End point*/
    ep_desc->ep_dir = PDC_EP_DIR_OUT;                   /* In endpoint */
    ep_desc->attributes = PDC_EP_CONTROL;               /* Control type*/
    ep_desc->max_pkt_size = 64;                                 /* Maximum packet size */

#if defined(CONFIG_USB_ISP1761_MSCD)
printk("EP_MSCD\n");
    ep_desc++;
    ep_desc->ep_num = 1;                                                /* Bulk out end point */
    ep_desc->ep_dir = PDC_EP_DIR_OUT;                   /* Out endpoint */
    ep_desc->attributes = PDC_EP_BULK;                  /* Transfer type*/
    ep_desc->max_pkt_size = 64;                                 /* Maximum packet size */

    ep_desc++;
    ep_desc->ep_num = 2;                                           /* Bulk In end point */
    ep_desc->ep_dir = PDC_EP_DIR_IN;                    /* In endpoint */
    ep_desc->attributes = PDC_EP_BULK;                  /* Transfer type*/
    ep_desc->max_pkt_size = 64;                                 /* Maximum packet size */
#endif
#if defined(CONFIG_USB_ISP1761_WOLFDEV) || defined(CONFIG_USB_ISP1761_WOLFDEV_MODULE)
printk("EP_WOLFDEV\n");
    ep_desc++;
    ep_desc->ep_num = 3;                                                /* Bulk out end point */
    ep_desc->ep_dir = PDC_EP_DIR_OUT;                   /* Out endpoint */
    ep_desc->attributes = PDC_EP_BULK;                  /* Transfer type*/
    ep_desc->max_pkt_size = 64;                                 /* Maximum packet size */

    ep_desc++;
    ep_desc->ep_num = 4;                                           /* Bulk In end point */
    ep_desc->ep_dir = PDC_EP_DIR_IN;                    /* In endpoint */
    ep_desc->attributes = PDC_EP_BULK;                  /* Transfer type*/
    ep_desc->max_pkt_size = 64;                                 /* Maximum packet size */
#endif

    bus_dev->pdc_config->num_eps = 1;   /* Control in and out */
    bus_dev->pdc_config->context = (unsigned long)bus_dev->pdc_config;
    bus_dev->pdc_config->notif = pdc_bus_dev_notif;

    result = pdc_configure_device(bus_dev->pdc_config);

    if(result==0) {
        ep_desc = bus_dev->pdc_config->ep_desc;

        bus_dev->pipe_desc.ep = ep_desc->ep_num;
        bus_dev->pipe_desc.ep_dir = ep_desc->ep_dir;
        bus_dev->pipe_desc.context = (unsigned long)ep_desc;
        bus_dev->pipe_desc.priv = (void*)bus_dev->pdc_config;
        bus_dev->pipe_desc.notify = pdc_bus_ctrl_pipe_notificaion;
        bus_dev->ctrl_pipe = pdc_open_pipe(&bus_dev->pipe_desc);

        if(bus_dev->ctrl_pipe == PDC_INV_PIPE_HANDLE) {
            result = -1;
        } else {
            result = 0;
            bus_dev->state = PDC_BUS_INIT;
            bus_dev->pdc_priv = pdc;
        }
    }

    return result;
} /* End of pdc_bus_init(void) */

void pdc_bus_deinit(void)
{
    dev_print("++ pdc_bus_deinit(void)\n");

    /* Close the control pipe */
    if(bus_dev->ctrl_pipe!= PDC_INV_PIPE_HANDLE){
        pdc_close_pipe(bus_dev->ctrl_pipe);
        bus_dev->ctrl_pipe= PDC_INV_PIPE_HANDLE;
    }

    /* Free the configuration data */
    if(bus_dev->pdc_config) kfree(bus_dev->pdc_config);
    bus_dev->pdc_config = NULL;
    bus_dev->pdc_priv = NULL;

    bus_dev->state = PDC_BUS_UNKNOWN;

    return;
} /* End of pdc_bus_deinit() */



void pdc_bus_dev_notif(unsigned long    context, unsigned long notif)
{
    struct list_head *tmp;

    dev_print("++ pdc_bus_dev_notif(context=%x,notif=%x)\n",context, notif);


    switch(notif) {
        case PDC_RESET:


            if(bus_dev->configuration) {

                bus_dev->configuration = 0;
                tmp = pdc_class_drv_list.next;
                /* Find the class driver that supports this request */
                while (tmp != &pdc_class_drv_list) {
                    struct pdc_class_drv *cd = list_entry(tmp,struct pdc_class_drv, driver_list);

                    tmp = tmp->next;
                    if(cd->set_config)  cd->set_config(cd->priv_data,bus_dev->configuration);
                }
                bus_dev->pdc_config->num_eps = 1;       /* Control */
                pdc_configure_device(bus_dev->pdc_config);
            }

            bus_dev->configuration = 0;
            bus_dev->state = PDC_BUS_DEFAULT;
            break;

        case PDC_RESUME:
            bus_dev->state = bus_dev->prev_state;
            break;

        case PDC_BUSTATUS:

            break;
        case PDC_SUSPEND:
            bus_dev->prev_state = bus_dev->state;
            bus_dev->state = PDC_BUS_SUSPENDED;
            break;

    }

#ifdef CONFIG_USB_OTG
    /* all kind of device notifications are passed to the OTG controller
     * (reset, suspend, resume)
     */
    if( bus_dev->otg && bus_dev->otg->otg_notif) {
        /* if(notif != PDC_SUSPEND) Suspend not masked */
        bus_dev->otg->otg_notif(bus_dev->otg->priv_data, notif,0);
    }
#endif /* CONFIG_USB_OTG */

    return;
} /* End of pdc_bus_dev_notif */


/*--------------------------------------------------------------*
 *           USB Class/Vendor specific functions
 *--------------------------------------------------------------*/

/* Make a configuration active */
void pdc_bus_set_configuration(unsigned char *command)
{
    int result;
    struct list_head *tmp;

    dev_print("++ set_configuration((command=%p)\n",command);

    if((bus_dev->state < PDC_BUS_ADDRESSED) || (command[0]&PDC_REQTYPE_DIR_MASK)) {
        /* You are not supposed to receive this in these states */
        pdc_bus_stall_control_pipe();
        return;
    }


    if(command[2] > 1) {

        /* Invalid configuration */
        /* Stall control in and out pipes */
        pdc_bus_stall_control_pipe();

    } else {
        if((bus_dev->configuration) || (command[2] == 0)) {
            /* Unconfigure the class drivers */

            tmp = pdc_class_drv_list.next;
            /* Find the class driver that supports this request */
            while (tmp != &pdc_class_drv_list) {
                struct pdc_class_drv *cd = list_entry(tmp,struct pdc_class_drv, driver_list);

                tmp = tmp->next;
                if(cd->set_config)      cd->set_config(cd->priv_data,0);
            }

            bus_dev->pdc_config->num_eps = 1;   /* Control */
            result = pdc_configure_device(bus_dev->pdc_config);
            bus_dev->state = PDC_BUS_ADDRESSED;
        }

        bus_dev->configuration = command[2];
        bus_dev->interface = bus_dev->alternate = 0;

        if(bus_dev->configuration) {
            /* Configure the class drivers */
            bus_dev->pdc_config->num_eps = 3;   /* Control + bulk */
            result = pdc_configure_device(bus_dev->pdc_config);
            bus_dev->state = PDC_BUS_CONFIGURED;

            tmp = pdc_class_drv_list.next;
            /* Find the class driver that supports this request */
            while (tmp != &pdc_class_drv_list) {
                struct pdc_class_drv *cd = list_entry(tmp,struct pdc_class_drv, driver_list);

                tmp = tmp->next;
                if(cd->set_config)      cd->set_config(cd->priv_data,bus_dev->configuration);
            }
        }

        pdc_notif_write_control_pipe(NULL, 0);

    }

    return;

} /* End of pdc_bus_set_configuration() */

void pdc_bus_set_interface(unsigned char *command)
{
    struct list_head *tmp;

    if((bus_dev->state > PDC_BUS_DEFAULT) && (command[2] < PDC_BUS_NUM_INTF) &&
            !(command[0]&PDC_REQTYPE_DIR_MASK)) {
        bus_dev->interface = command[2];
        bus_dev->alternate = command[4];

        tmp = pdc_class_drv_list.next;
        /* Find the class driver that supports this request */
        while (tmp != &pdc_class_drv_list) {
            struct pdc_class_drv *cd = list_entry(tmp,struct pdc_class_drv, driver_list);

            if(cd->set_intf)    cd->set_intf(cd->priv_data,bus_dev->interface, 
                    bus_dev->alternate);
            tmp = tmp->next;
        }
        pdc_notif_write_control_pipe(NULL, 0);
    } else {
        pdc_bus_stall_control_pipe();
    }

} /* End of pdc_bus_set_interface() */

void pdc_bus_get_status(__u8 *command)
{
    unsigned char reply[4];
    __u8        bmRequestType = command[0];
    pdc_pipe_handle_t           handle;
    __u8        reply_len = 2;

    if((bus_dev->state < PDC_BUS_ADDRESSED) || !(command[0]&PDC_REQTYPE_DIR_MASK)) {
        /* You are not supposed to receive this in these states */
        pdc_bus_stall_control_pipe();
        return;
    }

    /* Find request target */
    switch (bmRequestType&0x03) {
        case RECIP_DEVICE:          /* DEVICE */  
            reply[0] = PDC_GETSTATUS_BUS_POWERED;
            reply[1]=0;
            break;

        case RECIP_INTERFACE:       /* INTERFACE */  

            reply[0]=0;
            reply[1]=0;
            break;

        case RECIP_ENDPOINT: 
            /* reply[0] needs to be 1 if the endpoint
               referred to in command[3] is stalled,
               otherwise 0 */

            /* ep->pipe handle and context conversion */
            handle = pdc_usb_to_epindex((command[4]&0x0F),(command[4]&0x80));

            /* Check EP status */
            ctrl_pipe_opr.handle = handle;
            ctrl_pipe_opr.context = (unsigned long) bus_dev;
            ctrl_pipe_opr.opr = PDC_GET_PIPE_STATUS;
            pdc_pipe_operation(&ctrl_pipe_opr);

            reply[0] = (ctrl_pipe_opr.pipe_status & PDC_PIPE_STALL) ? 1 : 0;
            reply[1]=0;

            break;

        default:                        /* UNDEFINED */   
            /* Stall endpoints 0 & 1 */
            reply_len = 0;

            return;
    }

    /* Write this packet */
    pdc_notif_write_control_pipe(reply, reply_len);

    return;

} /* End of pdc_bus_get_status() */

__u16   pdc_read16(__u16 reg);
void pdc_write16(__u16  reg, __u16      data);


void pdc_bus_usb_feature(__u8   *command)
{
    __u8        bmRequestType = command[0];
    __u8        inv_state = PDC_BUS_ADDRESSED;


#if  1
    __u16                       uData;
    unsigned char * test_ptr;
    int                         length;

    /* ADDED for HSET TEST MODE */
    if((command[2]|(command[3]<<8)) == TEST_MODE)
    {
        if((command[4] == 0 ) )
        {
            switch(command[5])
            {

                case  TEST_J   :

                    pdc_notif_write_control_pipe(NULL, 0);
                    udelay(1000);
                    uData = TESTMODE_BIT_FORCEHS;
                    pdc_write16(TESTMODE_REGISTER, uData);
                    uData = TESTMODE_BIT_JSTATE | TESTMODE_BIT_FORCEHS;
                    pdc_write16(TESTMODE_REGISTER, uData);        
                    while(1);

                case  TEST_K  :
                    pdc_notif_write_control_pipe(NULL, 0);
                    udelay(1000);
                    uData = TESTMODE_BIT_FORCEHS;
                    pdc_write16(TESTMODE_REGISTER, uData);
                    uData = TESTMODE_BIT_KSTATE | TESTMODE_BIT_FORCEHS;
                    pdc_write16(TESTMODE_REGISTER, uData);        
                    while(1);

                case  TEST_SE0_NAK  :

                    pdc_notif_write_control_pipe(NULL, 0);
                    udelay(1000);       
                    uData = TESTMODE_BIT_FORCEHS;
                    pdc_write16(TESTMODE_REGISTER, uData);
                    uData = TESTMODE_BIT_SE0_NAK | TESTMODE_BIT_FORCEHS;
                    pdc_write16(TESTMODE_REGISTER, uData);        
                    while(1);
                case  TEST_PACKET  :
                    pdc_notif_write_control_pipe(NULL, 0);
                    udelay(1000);
                    length = 56;
                    test_ptr = Test_Packet;
                    pdc_notif_write_control_pipe(test_ptr, length);
                    uData = TESTMODE_BIT_FORCEHS;
                    pdc_write16(TESTMODE_REGISTER, uData);
                    uData = TESTMODE_BIT_PRBS | TESTMODE_BIT_FORCEHS;
                    pdc_write16(TESTMODE_REGISTER, uData);   
                    while(1);
                case  TEST_FORCE_ENABLE  :                                 
                    pdc_notif_write_control_pipe(NULL, 0);
                    udelay(1000);
                    while(1);

                default:
                    pdc_bus_stall_control_pipe();
                    return;
            }
        }
        else
        {
            pdc_bus_stall_control_pipe();
            return;
        }
    }

#endif



    /* 
     * For USB 2.0 Suppliment OTG 1.0 Specification,
     * Set feature will be accepted in default state also 
     */

    if(command[1]==SET_FEATURE) inv_state = PDC_BUS_DEFAULT;

    if((bus_dev->state < inv_state) || (command[0]&PDC_REQTYPE_DIR_MASK)) {
        /* You are not supposed to receive this in these states */
        printk("received SET_FEATURE in invalid state. STALLing\n");
        pdc_bus_stall_control_pipe();
        return;
    }

    switch (bmRequestType & 0x03) {

        case RECIP_DEVICE:

            if (command[2]==PDC_FEATURE_REMOTE_WAKEUP) {

                pdc_notif_write_control_pipe(NULL, 0);
            }
#ifdef CONFIG_USB_OTG
            else if ((command[2]==PDC_FEATURE_B_HNP_ENABLE)||
                    (command[2]==PDC_FEATURE_A_HNP_SUPPORT)||
                    (command[2]==PDC_FEATURE_A_ALTHNP_SUPPORT))
            {
                pdc_notif_write_control_pipe(NULL, 0);

                /*inform the OTG module of this */
                if( bus_dev->otg && bus_dev->otg->otg_notif) {
                    bus_dev->otg->otg_notif(bus_dev->otg->priv_data, PDC_SET_HNP,command[2]);
                }
            }
#endif /* CONFIG_USB_OTG */
            else {
                /* CF_WINDEX_FIX */
                /* Stall endpoints 0 & 1 */
                pdc_bus_stall_control_pipe();
            }

            break;

            /* INTERFACE */  
        case RECIP_INTERFACE:
            pdc_notif_write_control_pipe(NULL, 0);
            break;

            /* ENDPOINT */
        case RECIP_ENDPOINT:  
            {
                /* Find endpoint */
                pdc_pipe_handle_t handle;
                int stall=(command[1]==SET_FEATURE);

                handle = pdc_usb_to_epindex((command[4]&0x0F),(command[4]&0x80));
                dev_check("pdc: endpoint stall(%d)\n",handle);

                /* Set/clear endpoint stall flag */
                ctrl_pipe_opr.handle = handle;
                ctrl_pipe_opr.context = (unsigned long) bus_dev;
                if (stall) {
                    ctrl_pipe_opr.opr = PDC_PIPE_STALL;
                } else if(handle > 0x01) {
                    /* For Control In & Out already unstalled by
                       HW */
                    ctrl_pipe_opr.opr = PDC_PIPE_UNSTALL;
                }

                pdc_pipe_operation(&ctrl_pipe_opr);

                /* 0-byte ACK */
                pdc_notif_write_control_pipe(NULL, 0);
            }
            break;

            /* UNDEFINED */   
        default:              
            /* Stall endpoints 0 & 1 */
            printk("Error: recd undef SET/CLEAR_FEATURE reqtype=%x,cmd2=%d. Stalling\n", bmRequestType, command[2]);
            pdc_bus_stall_control_pipe();
            break;
    }

}


/* Deal with get_descriptor */
void pdc_bus_get_descriptor(__u8 *command)
{
    __u8        *desc_ptr = NULL;
    __u16       desc_len = 0, req_len;

    dev_print("++ get_descriptor(command=%d)\n",command[3]);

    if((bus_dev->state < PDC_BUS_ADDRESSED) && !(command[0]&PDC_REQTYPE_DIR_MASK)) {
        /* You are not supposed to receive this in these states */
        pdc_bus_stall_control_pipe();
        return;
    }


    switch( command[3]) {

        case PDC_DEV_DESC_TYPE:

            if(HS_Dev_flag == HIGH_SPEED)
                desc_ptr = HS_device_desc;
            else
                desc_ptr = device_desc;
            desc_len = sizeof(device_desc);

            break;

        case PDC_CONFIG_DESC_TYPE:

            if(command[2] < PDC_BUS_NUM_CONFIG) {

                if(HS_Dev_flag == HIGH_SPEED)
                    desc_ptr = HS_conf_desc;
                else
                    desc_ptr = conf_desc;
                desc_len = sizeof(conf_desc);
            }
            break;
            /* change for Compliance on 05-08-05 starts    */                    
        case PDC_CONFIG_QUALIFIER_TYPE:
            desc_len = sizeof(device_qualifier_desc);
            desc_ptr = device_qualifier_desc;
            break;
        case PDC_OTHER_DESC_TYPE:
            if(command[2] < PDC_BUS_NUM_CONFIG) {

                if(HS_Dev_flag == HIGH_SPEED){
                    desc_ptr = other_desc;
                    desc_len = sizeof(other_desc);
                }
                else{
                    desc_ptr = HS_other_desc;
                    desc_len = sizeof(HS_other_desc);
                }
            }   
            break;
            /* change ends         */
        case PDC_STRING_DESC_TYPE:

            switch(command[2]) {

                case 0:
                    desc_ptr = unicode_string;
                    desc_len = sizeof(unicode_string);

                    break;

                case 1:
                    desc_ptr = mfg_string;
                    desc_len = sizeof(mfg_string);

                    break;

                case 2:
                    desc_ptr = product_string;
                    desc_len = sizeof(product_string);

                    break;
                case 3:
                    desc_ptr = serial_string;
                    desc_len = sizeof(serial_string);

                default:
                    break;
            }
            break;

#ifdef CONFIG_USB_OTG
        case PDC_OTG_DESC_TYPE:
            desc_ptr = &conf_desc[PDC_CONFIG_DESC_LEN+PDC_INTF_DESC_LEN+2*PDC_EP_DESC_LEN];
            desc_len = desc_ptr[0];

            break;
#endif /* CONFIG_USB_OTG */

        default:
            break;
    }

    if(desc_ptr) {
        /* Get max length that remote end wants */
        req_len =command[6]|(command[7]<<8);
        if (desc_len > req_len ) {
            desc_len = req_len ;   
        }

        pdc_notif_write_control_pipe(desc_ptr, desc_len);

    } else {
        pdc_bus_stall_control_pipe();
    }

    return;
} /* End of pdc_bus_get_descriptor() */



int     pdc_notif_write_control_pipe(__u8       *buff, __u16    len)
{
    struct pdc_urb      *urb  = &bus_dev->ctrl_write_urb;

    if(!buff) len = 0;

    /* Fill the URB and submit it */
    pdc_fill_non_iso_urb(urb, bus_dev->ctrl_pipe, PDC_PIPE_CONTROL, PDC_OPR_WRITE, buff, len, NULL,NULL);
    return pdc_submit_urb(urb);
} /* End of pdc_notif_write_control_pipe() */

int     pdc_notif_read_control_pipe(__u8        *buff, __u16    len, void *callback)
{
    struct pdc_urb      *urb  = &bus_dev->ctrl_read_urb;

    if(!buff) len = 0;

    /* Fill the URB and submit it */
    pdc_fill_non_iso_urb(urb, bus_dev->ctrl_pipe, PDC_PIPE_CONTROL, PDC_OPR_READ, buff, len, ((void(*)(struct pdc_urb *))callback),NULL);
    return pdc_submit_urb(urb);
} /* End of pdc_nofif_read_control_pipe() */

void pdc_bus_stall_control_pipe(void)
{

    /* Stall Control Out Pipe */
    ctrl_pipe_opr.handle = bus_dev->ctrl_pipe;
    ctrl_pipe_opr.context = (unsigned long) bus_dev;
    ctrl_pipe_opr.opr = PDC_PIPE_STALL;
    pdc_pipe_operation(&ctrl_pipe_opr);

} /* End of pdc_bus_stall_control_pipe() */


int pdc_bus_ctrl_pipe_notificaion(unsigned long notif_type, void *priv,unsigned char    *cmd)
{
    __u8        RequestType;
    __u8        bmRequest ;
    int         result = -1;
    __u8        reply[2];
    struct list_head *tmp;
    __u8        address;

    if(notif_type == PDC_SETUP_COMMAND) {
        /* SETUP Command from the control pipe */
        RequestType = cmd[0] & 0x60;

        switch(RequestType) {

            case STANDARD_REQUEST:

                /* Process USB Standrad request */
                bmRequest = cmd[1];

                dev_print("++ pdc_bus_ctrl_pipe_notificaion: CTRL Req-type %x\n", bmRequest);

                switch (bmRequest) {
                    case GET_DESCRIPTOR:

                        pdc_bus_get_descriptor(cmd);
                        break;

                    case CLEAR_FEATURE:
                    case SET_FEATURE:
                        pdc_bus_usb_feature(cmd);
                        break;

                    case SET_ADDRESS:
                        /* 
                         * Set the address in the device controller
                         * and send the status back to the Host
                         */
                        if((bus_dev->state != PDC_BUS_DEFAULT && bus_dev->state != PDC_BUS_ADDRESSED) || (cmd[0]&PDC_REQTYPE_DIR_MASK)) {
                            pdc_bus_stall_control_pipe();
                        } else {

                            address = (cmd[2]&0x7F);
                            pdc_set_device_address((unsigned long)bus_dev->pdc_config,address);
                            pdc_notif_write_control_pipe(NULL, 0);

                            if(address)
                            {
                                bus_dev->state = PDC_BUS_ADDRESSED;
                            }
                            else 
                            {
                                bus_dev->state = PDC_BUS_DEFAULT;
                            }
                        }

                        break;

                    case SET_CONFIGURATION:

                        pdc_bus_set_configuration(cmd);

                        break;

                    case SET_INTERFACE:

                        pdc_bus_set_interface(cmd);

                        break;

                    case GET_CONFIGURATION:

                        /* 
                         * Send the current configuration, the status stage is taken care
                         * by the device controller
                         */
                        if((bus_dev->state >= PDC_BUS_DEFAULT) && (cmd[0]&PDC_REQTYPE_DIR_MASK)) {
                            reply[0] = bus_dev->configuration;
                            pdc_notif_write_control_pipe(reply, 1);
                        } else {
                            pdc_bus_stall_control_pipe();
                        }

                        break;

                    case GET_INTERFACE:

                        /* 
                         * Send the current configuration, the status stage is taken care
                         * by the device controller
                         */
                        if((bus_dev->state >= PDC_BUS_CONFIGURED) && (cmd[0]&PDC_REQTYPE_DIR_MASK)) {
                            reply[0] = bus_dev->interface;
                            pdc_notif_write_control_pipe(reply, 1);
                        } else {
                            /* You are not supposed to receive this in these states */
                            pdc_bus_stall_control_pipe();
                        }

                        break;

                    case GET_STATUS:

                        pdc_bus_get_status(cmd);

                        break;

                    default:

                        /* SET_DESCRIPTOR, SYNCH_FRAME TODO */
                        /* Other requests are not known to us */
                        pdc_bus_stall_control_pipe();
                        break;
                }


                break;

            case CLASS_REQUEST:
            case VENDOR_REQUEST:
                /* Process Class Vendor request */

                /* Find the class driver that supports this request */
                tmp = pdc_class_drv_list.next;
                while (tmp != &pdc_class_drv_list) {
                    struct pdc_class_drv *cd = list_entry(tmp,struct pdc_class_drv, driver_list);

                    tmp = tmp->next;
                    if(cd->class_vendor) {
                        /* Currently done for only one class driver */
                        result = cd->class_vendor(cd->priv_data, cmd);
                        break;
                    }
                }

                if(result == 0){
                    pdc_notif_write_control_pipe(NULL, 0);
                }

                if(result < 0){
                    /* Stall control in and out pipes */
                    pdc_bus_stall_control_pipe();
                }

                break;


            default:
                /* All other requests need to be stalled */
                dev_check(" stall from default_vendor \n");
                pdc_bus_stall_control_pipe();

                break;
        }

    }

    return 0;
} /* End of pdc_bus_ctrl_pipe_notifacaion() */


/*-----------------------------------------------------------------*
 *                     External OTG interface functions            *
 *-----------------------------------------------------------------*/

#ifdef CONFIG_USB_OTG
int     pdc_otg_register(pdc_otg_data_t *otg_data)
{

    if( bus_dev && otg_data ) {
        otg_data->dc_priv_data = (void*)bus_dev;
        bus_dev->otg = otg_data;
        printk(KERN_INFO __FILE__ ": Registered Driver usb-otg\n");
        return 0;
    }
    return -1;
}

void    pdc_otg_unregister(pdc_otg_data_t       *otg_data) 
{

    if(bus_dev && otg_data) {
        otg_data->dc_priv_data = NULL;
        bus_dev->otg = NULL;
        printk(KERN_INFO __FILE__ ": De-registered Driver usb-otg\n");
    }
}

void    pdc_otg_control(void *priv, unsigned long opr)
{
    struct list_head *tmp;

    pdc_dev_control(opr);

    switch(opr) {
        case PDC_ENABLE:
            bus_dev->state = PDC_BUS_ATTACHED;
            break;

        case PDC_DISABLE:
            bus_dev->state = PDC_BUS_INIT;

            if(bus_dev->configuration) {
                bus_dev->configuration = 0;
                /* Find the class driver that supports this request */
                tmp = pdc_class_drv_list.next;
                while (tmp != &pdc_class_drv_list) {
                    struct pdc_class_drv *cd = list_entry(tmp,struct pdc_class_drv, driver_list);

                    /* Currently done for only one class driver */
                    if(cd->set_config)  cd->set_config(cd->priv_data,bus_dev->configuration);
                    tmp = tmp->next;
                }
            }
            break;
    }

}
EXPORT_SYMBOL(pdc_otg_control);
EXPORT_SYMBOL(pdc_otg_register);
EXPORT_SYMBOL(pdc_otg_unregister);
#endif /* CONFIG_USB_OTG */

