/*********************************************************************
 * Philips USB device protocol and Configuration driver
 *
 * (c) 2002 Koninklijke Philips Electronics N.V., All rights reserved
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * File Name:   pdc_bus.h
 *
 * History:     
 *
 * Date                Author                  Comments
 * ---------------------------------------------------------------------
 * Nov 29 2005        Prabhakar Kalasani       Initial Creation
 *
 ***********************************************************************/

#ifndef __PDC_BUS_H__
#define __PDC_BUS_H__

#define PDC_CONFIG_PIPE_MPS                     PDC_CTRL_MPS
#define PDC_CONFIG_PIPE_HS_MPS                  0x200

#define PDC_BUS_NUM_CONFIG                      0x01
#define PDC_BUS_NUM_INTF                        0x01
#define PDC_BUS_NUM_STRINGS                     0x03
/*
 * Values of bus states 
 */
#define PDC_BUS_INIT                            0x00
#define PDC_BUS_ATTACHED                        0x01
#define PDC_BUS_DEFAULT                         0x02
#define PDC_BUS_ADDRESSED                       0x03
#define PDC_BUS_CONFIGURED                      0x04
#define PDC_BUS_SUSPENDED                       0x05
#define PDC_BUS_UNKNOWN                         0xFF

typedef struct pdc_bus {
    __u8        state;
    __u8        prev_state;
    __u8        configuration;
    __u8        interface;
    __u8        alternate;
    struct pdc_config_dev       *pdc_config;
    pdc_pipe_handle_t   ctrl_pipe;              /* Control pipe handle */
    struct pdc_pipe_desc        pipe_desc;              /* Control pipe descriptor */
    struct pdc_urb              ctrl_write_urb;         /* Control pipe write URB */
    struct pdc_urb              ctrl_read_urb;          /* Control pipe read URB */
    void                        *pdc_priv;              /* Device Controller data structure */
#ifdef CONFIG_USB_OTG
    pdc_otg_data_t *otg;
#endif /* CONFIG_USB_OTG */
} pdc_bus_t;

struct pdc_bus_ctrlrequest {
    __u8 bRequestType;
    __u8 bRequest;
    __u16 wValue;
    __u16 wIndex;
    __u16 wLength;
} __attribute__ ((packed));

/*Define for Compliance */
#define  TEST_MODE                              0x02
#define  ENDPOINT_HALT                          0
#define  TEST_J                                 1
#define  TEST_K                                 2
#define  TEST_SE0_NAK                           3
#define  TEST_PACKET                            4
#define  TEST_FORCE_ENABLE                      5

#endif /* __PDC_BUS_H__ */
