/**********************************************************************
 * Philips ISP1181 device controller
 *
 * (c) 2002 Koninklijke Philips Electronics N.V., All rights reserved
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * File Name:   usb_pdc.h
 *
 * Refering linux kernel version 2.6.9
 *
 * History:     
 *
 * Date                Author                  Comments
 * ----------------------------------------------------------------------
 * Nov 29 2005        Prabhakar Kalasani        Initial Creation
 *
 ************************************************************************/
#ifndef __USB_PDC_H__
#define __USB_PDC_H__

#include "pdc_intf.h"

#define DRIVER_AUTHOR           "Philips Semiconductors"
#define DRIVER_DESC             "ISP1761 Peripheral Controller Driver"
#define PDC_DRIVER_VERSION      "1.0"
#define DRIVER_NAME             "usb-pdc"

#define CONTROL_EP                              0x00
#define ISOC_EP                                 0x01
#define BULK_EP                                 0x02
#define INT_EP                                  0x03

#define DIR_IN                                  0x01
#define DIR_OUT                                 0x00

#define AVAILABLE                               0x00
#define IN_USE                                  0x01
#define STALL                                   0x02

/* Register & bitsets */
#define ADDRESS_REGISTER                        0x0200
#define ADDRESS_DEVEN                           0x80

#define MODE_REGISTER                           0x020C
#define MODE_SOFTCT                             0x00
#define MODE_WKUPCS                             0x04
#define MODE_GLINTENA                           0x08
#define MODE_SFRESET                            0x10
#define MODE_GOSUSP                             0x20
#define MODE_SNDRSU                             0x40
#define MODE_CLKAON                             0x80
#define DMA_CLKON                               0x200


#define INT_CONFIG_REGISTER                     0x0210
#define CONFIG_INTPOL_L                         0x00
#define CONFIG_INTPOL_H                         0x01
#define CONFIG_INTLVL_L                         0x00
#define CONFIG_INTLVL_P                         0x02
#define CONFIG_INT_DBG_OUTACK_NAK_NYET          0x00
#define CONFIG_INT_DBG_OUT_ACK_NYET             0x04
#define CONFIG_INT_DBG_OUT_ACK_NYET_FNAK        0x08
#define CONFIG_INT_DBG_IN_ACK_NAK               0x00
#define CONFIG_INT_DBG_IN_ACK                   0x10
#define CONFIG_INT_DBG_IN_ACK_FNAK              0x20
#define CONFIG_INT_DBG_ACK_NAK                  0x00
#define CONFIG_INT_DBG_ACK                      0x40
#define CONFIG_INT_ACK_FNAK                     0x80
#define DEFAULT_INT_CONFIG                      (CONFIG_INT_DBG_ACK |   CONFIG_INT_DBG_IN_ACK|  CONFIG_INT_DBG_OUT_ACK_NYET )    //0x54


#define INT_ENABLE_REGISTER                     0x0214
#define IE_RST                                  0x00000001
#define IE_SOF                                  0x00000002
#define IE_PSOF                                 0x00000004
#define IE_SUSP                                 0x00000008
#define IE_RESM                                 0x00000010
#define IE_HS_STA                               0x00000020
#define IE_DMA                                  0x00000040
#define IE_VBUS                                 0x00000080
#define IE_EP0SETUP                             0x00000100
#define IE_EP0RX                                0x00000400
#define IE_EP0TX                                0x00000800
#define IE_EP1RX                                0x00001000
#define IE_EP1TX                                0x00002000
#define IE_EP2RX                                0x00004000
#define IE_EP2TX                                0x00008000
#define IE_EP3RX                                0x00010000
#define IE_EP3TX                                0x00020000
#define IE_EP4RX                                0x00040000
#define IE_EP4TX                                0x00080000
#define IE_EP5RX                                0x00100000
#define IE_EP5TX                                0x00200000
#define IE_EP6RX                                0x00400000
#define IE_EP6TX                                0x00800000
#define IE_EP7RX                                0x01000000
#define IE_EP7TX                                0x02000000
#define IE_NON_CTRL_EP_MASK                     (IE_EP1RX | IE_EP1TX | IE_EP2RX | IE_EP2TX | IE_EP3RX | IE_EP3TX | IE_EP4RX | IE_EP4TX | IE_EP5RX | IE_EP5TX | IE_EP6RX | IE_EP6TX | IE_EP7RX | IE_EP7TX)
#define IE_BASIC_CTRL_EP_MASK                   (IE_EP0RX | IE_EP0TX | IE_EP0SETUP | IE_VBUS | IE_HS_STA | IE_SUSP |IE_RST | IE_RESM) 


#define ENDPT_INDEX_REGISTER                    0x022c
#define EP0SETUP                                0x20
#define EP0RX                                   0x00
#define EP0TX                                   0x01
#define EP1RX                                   0x02
#define EP1TX                                   0x03
#define EP2RX                                   0x04
#define EP2TX                                   0x05
#define EP3RX                                   0x06
#define EP3TX                                   0x07
#define EP4RX                                   0x08
#define EP4TX                                   0x09
#define EP5RX                                   0x0a
#define EP5TX                                   0x0b
#define EP6RX                                   0x0c
#define EP6TX                                   0x0d
#define EP7RX                                   0x0e
#define EP7TX                                   0x0f

#define CTRLFN_REGISTER                         0x0228
#define CNTRLFN_STALL                           0x01
#define CNTRLFN_STATUS_NAK                      0x00
#define CNTRLFN_STATUS_EM_ACK                   0x02
#define CNTRLFN_VENDP                           0x08
#define CNTRLFN_CLBUF                           0x10
#define CNTRLFN_DSEN                            0x04

#define DATAPORT_REGISTER                       0x0220

#define BUFFER_STATUS_REG                       0x021E
#define BUF0_FILLED                             0x01
#define BUF1_FILLED                             0x02
#define BUF1_AND_2_FILLED                       0x03

#define BUFFER_LGTH_REGISTER                    0x021C

#define MAXP_SIZE_REGISTER                      0x0204
#define MAXP_SIZE_NTRANS_1                      0x0000
#define MAXP_SIZE_NTRANS_2                      0x0800
#define MAXP_SIZE_NTRANS_3                      0x1800

#define EP_TYP_REGISTER                         0x0208
#define EP_TYP_ENDPTY_ISO                       0x0001
#define EP_TYP_ENDPTY_BLK                       0x0002
#define EP_TYP_ENDPTY_INT                       0x0003
#define EP_TYP_DBLBUF                           0x0004
#define EP_TYP_ENABLE                           0x0008
#define EP_TYP_NOEMPKT                          0x0010

#define EPCONFIG_EPDIR                          0x0000

#define INTERRUPT_REGISTER                      0x0218
#define INT_REG_BRESET                          0x00000001
#define INT_REG_SOF                             0x00000002
#define INT_REG_PSOF                            0x00000004
#define INT_REG_SUSP                            0x00000008
#define INT_REG_RESUME                          0x00000010
#define INT_REG_HS_STAT                         0x00000020
#define INT_REG_DMA                             0x00000040
#define INT_REG_EP0SETUP                        0x00000100
#define INT_REG_EP0_RX                          0x00000400
#define INT_REG_EP0_TX                          0x00000800
#define INT_REG_EP1_RX                          0x00001000
#define INT_REG_EP1_TX                          0x00002000
#define INT_REG_EP2_RX                          0x00004000
#define INT_REG_EP2_TX                          0x00008000
#define INT_REG_EP3_RX                          0x00010000
#define INT_REG_EP3_TX                          0x00020000
#define INT_REG_EP4_RX                          0x00040000
#define INT_REG_EP4_TX                          0x00080000
#define INT_REG_EP5_RX                          0x00100000
#define INT_REG_EP5_TX                          0x00200000
#define INT_REG_EP6_RX                          0x00400000
#define INT_REG_EP6_TX                          0x00800000
#define INT_REG_EP7_RX                          0x01000000
#define INT_REG_EP7_TX                          0x02000000


#define DMA_CMD_REGISTER                    	0x0230
#define DMA_EP_INDX_REGISTER                    0x0258
#define CHIPID_REGISTER                         0x0270
#define FRAME_NUMBER_REGISTER                   0x0274
#define SCRATCH_REGISTER                        0x0278

#define UNLOCK_DEVICE_REG                       0x027C
#define DEFAULT_INT_PULSE_SIZE                  0x1E
#define INT_PULSE_SIZE_REG                      0x0280
#define TESTMODE_REGISTER                       0x0284
#define TESTMODE_BIT_SE0_NAK                    0x01
#define TESTMODE_BIT_JSTATE                     0x02
#define TESTMODE_BIT_KSTATE                     0x04
#define TESTMODE_BIT_PRBS                       0x08
#define TESTMODE_BIT_FORCEFS                    0x10
#define TESTMODE_BIT_FORCEHS                    0x80

/*
#define isp1761_vendor_id                       0x0A1B
#define isp1761_product_id                      0x0001

#define isp1761_product_string                  { 22,3,'V',0, 'i',0, 's',0, 'u',0, 'a',0, 'l',0, 'i',0, 'z',0, 'e',0,'r',0 }

#define isp1761_mfg_string                      { 22,3,'W',0, 'o',0, 'l',0, 'f',0, 'V',0, 'i',0, 's',0,'i',0, 'o',0, 'n',0 }
*/

#define isp1761_vendor_id                       0x04CC
#define isp1761_product_id                      0x1b41 /*0x1A64 */
#define isp1761_product_string                  { 68,3,'1',0, '7',0, '6',0, '2',0, '-',0, 'O',0, 'T',0, 'G',0, ' ',0,'M',0, 'A',0, 'S',0, 'S',0, ' ',0,'S',0, 't',0, 'o',0, 'r',0, 'a',0, 'g',0, 'e',0, ' ',0,'P',0, 'X',0, 'A',0, '-',0, '2',0, '5',0, '0',0,' ',0, 'K',0, 'i',0, 't',0 }

#define isp1761_mfg_string                      { 46,3,'P',0, 'h',0, 'i',0, 'l',0, 'i',0, 'p',0, 's',0,' ',0, 'S',0, 'e',0, 'm',0, 'i',0,'c',0, 'o',0, 'n',0, 'd',0, 'u',0, 'c',0, 't',0, 'o',0, 'r',0, 's',0 }

#define HIGH_SPEED                              1
#define FULL_SPEED                              0


/*-----------------------------------------------------------------*
 *       pipe data structure                                       *
 *-----------------------------------------------------------------*/
#define PDC_MAX_EPS                             0x10

#define pdc_usb_to_epindex(ep,dir)              (dir == 0) ? (ep << 1) : ((ep << 1)+1)
#define pdc_usb_to_epreg(ep,dir)                (((ep==0) && (dir==0)) ? 0 : (ep+1))
#define pdc_epreg_to_usb(epreg)                 ((epreg<2) ? 0 : (epreg-1))
#define pdc_pipe_to_epreg(pipe)                 (pipe)
#define pdc_epreg_to_pipe(epreg)                (epreg)

#define PDC_PIPE_UNCONFIG                       0x00
#define         PDC_PIPE_CONFIG                 0x01
#define         PDC_PIPE_OPEN                   0x02

typedef struct pdc_pipe {
    __u8                                        pipe;                   /* pipe number 0-15 */
    __u8                                        ep_state;               /* State of the endpoint */
    __u8                                        txrx_idle;              /* tx, rx idle flag */
    __u16                                       ep_status;

    struct pdc_urb                      *urb;                   /* USB request block list */
    struct pdc_ep_desc          *ep_desc;               /* End point descriptor */
    struct pdc_pipe_desc        *pipe_desc;             /* Pipe descriptor */
} pdc_pipe_t;

/* USB device: one in, one out, and various stats */
typedef struct pdc_dev
{
    /* Transmit buffer for this endpoint */
    struct isp1761_dev  *dev;
    /* Configuration driver information */
    struct pdc_config_dev *pdc_bus;
    /* pdc pipe data structure */
    struct pdc_pipe     *ep_pipes;
}pdc_dev_t;

#endif /* __USB_PDC_H__ */


